@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine, Bytecode Visualisation                                  |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2016 The PHP Group                                |
+   | Copyright (c) 1998-2017 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -393,9 +393,9 @@ static void zend_dump_op(const zend_op_array *op_array, const zend_basic_block *
 	}
 
 	if (!b) {
-		len = fprintf(stderr, "L%u:", (uint32_t)(opline - op_array->opcodes));
+		len = fprintf(stderr, "L%u (%u):", (uint32_t)(opline - op_array->opcodes), opline->lineno);
 	}
-	fprintf(stderr, "%*c", 8-len, ' ');
+	fprintf(stderr, "%*c", 12-len, ' ');
 
 	if (!ssa || !ssa->ops || ssa->ops[opline - op_array->opcodes].result_use < 0) {
 		if (opline->result_type & (IS_CV|IS_VAR|IS_TMP_VAR)) {
@@ -519,9 +519,6 @@ static void zend_dump_op(const zend_op_array *op_array, const zend_basic_block *
 			}
 		}
 		if (ZEND_VM_EXT_ISSET & flags) {
-		    if (opline->extended_value & ZEND_QUICK_SET) {
-				fprintf(stderr, " (quick)");
-		    }
 			if (opline->extended_value & ZEND_ISSET) {
 				fprintf(stderr, " (isset)");
 			} else if (opline->extended_value & ZEND_ISEMPTY) {
@@ -581,7 +578,28 @@ static void zend_dump_op(const zend_op_array *op_array, const zend_basic_block *
 	}
 
 	if (opline->op2_type == IS_CONST) {
-		zend_dump_const(CRT_CONSTANT_EX(op_array, opline->op2, (dump_flags & ZEND_DUMP_RT_CONSTANTS)));
+		zval *op = CRT_CONSTANT_EX(op_array, opline->op2, (dump_flags & ZEND_DUMP_RT_CONSTANTS));
+		if (opline->opcode == ZEND_SWITCH_LONG || opline->opcode == ZEND_SWITCH_STRING) {
+			HashTable *jumptable = Z_ARRVAL_P(op);
+			zend_string *key;
+			zend_ulong num_key;
+			zval *zv;
+			ZEND_HASH_FOREACH_KEY_VAL(jumptable, num_key, key, zv) {
+				if (key) {
+					fprintf(stderr, " \"%s\":", ZSTR_VAL(key));
+				} else {
+					fprintf(stderr, " " ZEND_LONG_FMT ":", num_key);
+				}
+				if (b) {
+					fprintf(stderr, " BB%d,", b->successors[n++]);
+				} else {
+					fprintf(stderr, " L%u,", (uint32_t)ZEND_OFFSET_TO_OPLINE_NUM(op_array, opline, Z_LVAL_P(zv)));
+				}
+			} ZEND_HASH_FOREACH_END();
+			fprintf(stderr, " default:");
+		} else {
+			zend_dump_const(op);
+		}
 	} else if (opline->op2_type & (IS_CV|IS_VAR|IS_TMP_VAR)) {
 		if (ssa && ssa->ops) {
 			int ssa_var_num = ssa->ops[opline - op_array->opcodes].op2_use;
@@ -654,7 +672,6 @@ static void zend_dump_op(const zend_op_array *op_array, const zend_basic_block *
 static void zend_dump_block_info(const zend_cfg *cfg, int n, uint32_t dump_flags)
 {
 	zend_basic_block *b = cfg->blocks + n;
-	int printed = 0;
 
 	fprintf(stderr, "BB%d:", n);
 	if (b->flags & ZEND_BB_START) {
@@ -690,7 +707,7 @@ static void zend_dump_block_info(const zend_cfg *cfg, int n, uint32_t dump_flags
 	if (b->flags & ZEND_BB_KILL_VAR) {
 		fprintf(stderr, " kill_var");
 	}
-	if (!(dump_flags & ZEND_DUMP_HIDE_UNREACHABLE) & !(b->flags & ZEND_BB_REACHABLE)) {
+	if (!(dump_flags & ZEND_DUMP_HIDE_UNREACHABLE) && !(b->flags & ZEND_BB_REACHABLE)) {
 		fprintf(stderr, " unreachable");
 	}
 	if (b->flags & ZEND_BB_LOOP_HEADER) {
@@ -717,14 +734,12 @@ static void zend_dump_block_info(const zend_cfg *cfg, int n, uint32_t dump_flags
 		fprintf(stderr, ")\n");
 	}
 
-	if (b->successors[0] != -1) {
+	if (b->successors_count > 0) {
+		int s;
 		fprintf(stderr, "    ; to=(BB%d", b->successors[0]);
-		printed = 1;
-		if (b->successors[1] != -1) {
-			fprintf(stderr, ", BB%d", b->successors[1]);
+		for (s = 1; s < b->successors_count; s++) {
+			fprintf(stderr, ", BB%d", b->successors[s]);
 		}
-	}
-	if (printed) {
 		fprintf(stderr, ")\n");
 	}
 
