@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2017 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,14 +16,14 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: iptc.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id$ */
 
 /*
  * Functions to parse & compse IPTC data.
  * PhotoShop >= 3.0 can read and write textual data to JPEG files.
  * ... more to come .....
- * 
- * i know, parts of this is now duplicated in image.c 
+ *
+ * i know, parts of this is now duplicated in image.c
  * but in this case i think it's okay!
  */
 
@@ -31,13 +31,22 @@
  * TODO:
  *  - add IPTC translation table
  */
- 
+
 #include "php.h"
 #include "php_iptc.h"
 #include "ext/standard/head.h"
 
 #include <sys/stat.h>
 
+#ifdef PHP_WIN32
+# include "win32/php_stdint.h"
+#else
+# if HAVE_INTTYPES_H
+#  include <inttypes.h>
+# elif HAVE_STDINT_H
+#  include <stdint.h>
+# endif
+#endif
 
 /* some defines for the different JPEG block types */
 #define M_SOF0  0xC0            /* Start Of Frame N */
@@ -75,8 +84,8 @@
 
 /* {{{ php_iptc_put1
  */
-static int php_iptc_put1(FILE *fp, int spool, unsigned char c, unsigned char **spoolbuf TSRMLS_DC)
-{ 
+static int php_iptc_put1(FILE *fp, int spool, unsigned char c, unsigned char **spoolbuf)
+{
 	if (spool > 0)
 		PUTC(c);
 
@@ -88,8 +97,8 @@ static int php_iptc_put1(FILE *fp, int spool, unsigned char c, unsigned char **s
 
 /* {{{ php_iptc_get1
  */
-static int php_iptc_get1(FILE *fp, int spool, unsigned char **spoolbuf TSRMLS_DC)
-{ 	
+static int php_iptc_get1(FILE *fp, int spool, unsigned char **spoolbuf)
+{
 	int c;
 	char cc;
 
@@ -110,9 +119,9 @@ static int php_iptc_get1(FILE *fp, int spool, unsigned char **spoolbuf TSRMLS_DC
 
 /* {{{ php_iptc_read_remaining
  */
-static int php_iptc_read_remaining(FILE *fp, int spool, unsigned char **spoolbuf TSRMLS_DC)
+static int php_iptc_read_remaining(FILE *fp, int spool, unsigned char **spoolbuf)
 {
-  	while (php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC) != EOF) continue;
+  	while (php_iptc_get1(fp, spool, spoolbuf) != EOF) continue;
 
 	return M_EOI;
 }
@@ -120,21 +129,21 @@ static int php_iptc_read_remaining(FILE *fp, int spool, unsigned char **spoolbuf
 
 /* {{{ php_iptc_skip_variable
  */
-static int php_iptc_skip_variable(FILE *fp, int spool, unsigned char **spoolbuf TSRMLS_DC)
-{ 
+static int php_iptc_skip_variable(FILE *fp, int spool, unsigned char **spoolbuf)
+{
 	unsigned int  length;
 	int c1, c2;
 
-    if ((c1 = php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC)) == EOF) return M_EOI;
+    if ((c1 = php_iptc_get1(fp, spool, spoolbuf)) == EOF) return M_EOI;
 
-    if ((c2 = php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC)) == EOF) return M_EOI;
+    if ((c2 = php_iptc_get1(fp, spool, spoolbuf)) == EOF) return M_EOI;
 
 	length = (((unsigned char) c1) << 8) + ((unsigned char) c2);
 
 	length -= 2;
 
 	while (length--)
-		if (php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC) == EOF) return M_EOI;
+		if (php_iptc_get1(fp, spool, spoolbuf) == EOF) return M_EOI;
 
 	return 0;
 }
@@ -142,29 +151,29 @@ static int php_iptc_skip_variable(FILE *fp, int spool, unsigned char **spoolbuf
 
 /* {{{ php_iptc_next_marker
  */
-static int php_iptc_next_marker(FILE *fp, int spool, unsigned char **spoolbuf TSRMLS_DC)
+static int php_iptc_next_marker(FILE *fp, int spool, unsigned char **spoolbuf)
 {
     int c;
 
     /* skip unimportant stuff */
 
-    c = php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC);
+    c = php_iptc_get1(fp, spool, spoolbuf);
 
 	if (c == EOF) return M_EOI;
 
     while (c != 0xff) {
-        if ((c = php_iptc_get1(fp, spool, spoolbuf TSRMLS_CC)) == EOF)
+        if ((c = php_iptc_get1(fp, spool, spoolbuf)) == EOF)
             return M_EOI; /* we hit EOF */
     }
 
     /* get marker byte, swallowing possible padding */
     do {
-        c = php_iptc_get1(fp, 0, 0 TSRMLS_CC);
+        c = php_iptc_get1(fp, 0, 0);
 		if (c == EOF)
             return M_EOI;       /* we hit EOF */
 		else
 		if (c == 0xff)
-			php_iptc_put1(fp, spool, (unsigned char)c, spoolbuf TSRMLS_CC);
+			php_iptc_put1(fp, spool, (unsigned char)c, spoolbuf);
     } while (c == 0xff);
 
     return (unsigned int) c;
@@ -178,72 +187,76 @@ static char psheader[] = "\xFF\xED\0\0Photoshop 3.0\08BIM\x04\x04\0\0\0\0";
 PHP_FUNCTION(iptcembed)
 {
 	char *iptcdata, *jpeg_file;
-	int iptcdata_len, jpeg_file_len;
-	long spool = 0;
+	size_t iptcdata_len, jpeg_file_len;
+	zend_long spool = 0;
 	FILE *fp;
-	unsigned int marker, done = 0, inx;
-	unsigned char *spoolbuf = NULL, *poi = NULL;
-	struct stat sb;
+	unsigned int marker, done = 0;
+	size_t inx;
+	zend_string *spoolbuf = NULL;
+	unsigned char *poi = NULL;
+	zend_stat_t sb;
 	zend_bool written = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|l", &iptcdata, &iptcdata_len, &jpeg_file, &jpeg_file_len, &spool) != SUCCESS) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_STRING(iptcdata, iptcdata_len)
+		Z_PARAM_PATH(jpeg_file, jpeg_file_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(spool)
+	ZEND_PARSE_PARAMETERS_END();
 
-	if (strlen(jpeg_file) != jpeg_file_len) {
+	if (php_check_open_basedir(jpeg_file)) {
 		RETURN_FALSE;
 	}
 
-	if (PG(safe_mode) && (!php_checkuid(jpeg_file, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
-		RETURN_FALSE;
-	}
-
-	if (php_check_open_basedir(jpeg_file TSRMLS_CC)) {
+	if (iptcdata_len >= SIZE_MAX - sizeof(psheader) - 1025) {
+		php_error_docref(NULL, E_WARNING, "IPTC data too large");
 		RETURN_FALSE;
 	}
 
 	if ((fp = VCWD_FOPEN(jpeg_file, "rb")) == 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to open %s", jpeg_file);
+		php_error_docref(NULL, E_WARNING, "Unable to open %s", jpeg_file);
 		RETURN_FALSE;
 	}
 
 	if (spool < 2) {
-		fstat(fileno(fp), &sb);
+		zend_fstat(fileno(fp), &sb);
 
-		poi = spoolbuf = safe_emalloc(1, iptcdata_len + sizeof(psheader) + sb.st_size + 1024, 1);
+		spoolbuf = zend_string_safe_alloc(1, iptcdata_len + sizeof(psheader) + 1024 + 1, sb.st_size, 0);
+		poi = (unsigned char*)ZSTR_VAL(spoolbuf);
 		memset(poi, 0, iptcdata_len + sizeof(psheader) + sb.st_size + 1024 + 1);
-	} 
+	}
 
-	if (php_iptc_get1(fp, spool, poi?&poi:0 TSRMLS_CC) != 0xFF) {
+	if (php_iptc_get1(fp, spool, poi?&poi:0) != 0xFF) {
 		fclose(fp);
 		if (spoolbuf) {
-			efree(spoolbuf);
+			zend_string_free(spoolbuf);
 		}
 		RETURN_FALSE;
 	}
 
-	if (php_iptc_get1(fp, spool, poi?&poi:0 TSRMLS_CC) != 0xD8) {
+	if (php_iptc_get1(fp, spool, poi?&poi:0) != 0xD8) {
 		fclose(fp);
 		if (spoolbuf) {
-			efree(spoolbuf);
+			zend_string_free(spoolbuf);
 		}
 		RETURN_FALSE;
 	}
 
 	while (!done) {
-		marker = php_iptc_next_marker(fp, spool, poi?&poi:0 TSRMLS_CC);
+		marker = php_iptc_next_marker(fp, spool, poi?&poi:0);
 
 		if (marker == M_EOI) { /* EOF */
 			break;
-		} else if (marker != M_APP13) { 
-			php_iptc_put1(fp, spool, (unsigned char)marker, poi?&poi:0 TSRMLS_CC);
+		} else if (marker != M_APP13) {
+			php_iptc_put1(fp, spool, (unsigned char)marker, poi?&poi:0);
 		}
 
 		switch (marker) {
 			case M_APP13:
 				/* we are going to write a new APP13 marker, so don't output the old one */
-				php_iptc_skip_variable(fp, 0, 0 TSRMLS_CC);    
-				php_iptc_read_remaining(fp, spool, poi?&poi:0 TSRMLS_CC);
+				php_iptc_skip_variable(fp, 0, 0);
+				fgetc(fp); /* skip already copied 0xFF byte */
+				php_iptc_read_remaining(fp, spool, poi?&poi:0);
 				done = 1;
 				break;
 
@@ -256,7 +269,7 @@ PHP_FUNCTION(iptcembed)
 				}
 				written = 1;
 
-				php_iptc_skip_variable(fp, spool, poi?&poi:0 TSRMLS_CC);
+				php_iptc_skip_variable(fp, spool, poi?&poi:0);
 
 				if (iptcdata_len & 1) {
 					iptcdata_len++; /* make the length even */
@@ -266,25 +279,25 @@ PHP_FUNCTION(iptcembed)
 				psheader[ 3 ] = (iptcdata_len+28)&0xff;
 
 				for (inx = 0; inx < 28; inx++) {
-					php_iptc_put1(fp, spool, psheader[inx], poi?&poi:0 TSRMLS_CC);
+					php_iptc_put1(fp, spool, psheader[inx], poi?&poi:0);
 				}
 
-				php_iptc_put1(fp, spool, (unsigned char)(iptcdata_len>>8), poi?&poi:0 TSRMLS_CC);
-				php_iptc_put1(fp, spool, (unsigned char)(iptcdata_len&0xff), poi?&poi:0 TSRMLS_CC);
+				php_iptc_put1(fp, spool, (unsigned char)(iptcdata_len>>8), poi?&poi:0);
+				php_iptc_put1(fp, spool, (unsigned char)(iptcdata_len&0xff), poi?&poi:0);
 
 				for (inx = 0; inx < iptcdata_len; inx++) {
-					php_iptc_put1(fp, spool, iptcdata[inx], poi?&poi:0 TSRMLS_CC);
+					php_iptc_put1(fp, spool, iptcdata[inx], poi?&poi:0);
 				}
 				break;
 
-			case M_SOS:								
+			case M_SOS:
 				/* we hit data, no more marker-inserting can be done! */
-				php_iptc_read_remaining(fp, spool, poi?&poi:0 TSRMLS_CC);
+				php_iptc_read_remaining(fp, spool, poi?&poi:0);
 				done = 1;
 				break;
 
 			default:
-				php_iptc_skip_variable(fp, spool, poi?&poi:0 TSRMLS_CC);
+				php_iptc_skip_variable(fp, spool, poi?&poi:0);
 				break;
 		}
 	}
@@ -292,7 +305,8 @@ PHP_FUNCTION(iptcembed)
 	fclose(fp);
 
 	if (spool < 2) {
-		RETVAL_STRINGL(spoolbuf, poi - spoolbuf, 0);
+		spoolbuf = zend_string_truncate(spoolbuf, poi - (unsigned char*)ZSTR_VAL(spoolbuf), 0);
+		RETURN_NEW_STR(spoolbuf);
 	} else {
 		RETURN_TRUE;
 	}
@@ -303,20 +317,21 @@ PHP_FUNCTION(iptcembed)
    Parse binary IPTC-data into associative array */
 PHP_FUNCTION(iptcparse)
 {
-	unsigned int inx = 0, len, tagsfound = 0;
-	unsigned char *buffer, recnum, dataset, key[ 16 ];
-	char *str;
-	int str_len;
-	zval *values, **element;
+	size_t inx = 0, len;
+	unsigned int tagsfound = 0;
+	unsigned char *buffer, recnum, dataset;
+	char *str, key[16];
+	size_t str_len;
+	zval values, *element;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) != SUCCESS) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_STRING(str, str_len)
+	ZEND_PARSE_PARAMETERS_END();
 
 	buffer = (unsigned char *)str;
 
 	while (inx < str_len) { /* find 1st tag */
-		if ((buffer[inx] == 0x1c) && ((buffer[inx+1] == 0x01) || (buffer[inx+1] == 0x02))){ 
+		if ((buffer[inx] == 0x1c) && ((buffer[inx+1] == 0x01) || (buffer[inx+1] == 0x02))){
 			break;
 		} else {
 			inx++;
@@ -326,8 +341,8 @@ PHP_FUNCTION(iptcparse)
 	while (inx < str_len) {
 		if (buffer[ inx++ ] != 0x1c) {
 			break;   /* we ran against some data which does not conform to IPTC - stop parsing! */
-		} 
-		
+		}
+
 		if ((inx + 4) >= str_len)
 			break;
 
@@ -335,32 +350,34 @@ PHP_FUNCTION(iptcparse)
 		recnum = buffer[ inx++ ];
 
 		if (buffer[ inx ] & (unsigned char) 0x80) { /* long tag */
-			len = (((long) buffer[ inx + 2 ]) << 24) + (((long) buffer[ inx + 3 ]) << 16) + 
-				  (((long) buffer[ inx + 4 ]) <<  8) + (((long) buffer[ inx + 5 ]));
+			if((inx+6) >= str_len) {
+				break;
+			}
+			len = (((zend_long) buffer[ inx + 2 ]) << 24) + (((zend_long) buffer[ inx + 3 ]) << 16) +
+				  (((zend_long) buffer[ inx + 4 ]) <<  8) + (((zend_long) buffer[ inx + 5 ]));
 			inx += 6;
 		} else { /* short tag */
 			len = (((unsigned short) buffer[ inx ])<<8) | (unsigned short)buffer[ inx+1 ];
 			inx += 2;
 		}
 
-		snprintf(key, sizeof(key), "%d#%03d", (unsigned int) dataset, (unsigned int) recnum);
-
 		if ((len > str_len) || (inx + len) > str_len) {
 			break;
 		}
 
+		snprintf(key, sizeof(key), "%d#%03d", (unsigned int) dataset, (unsigned int) recnum);
+
 		if (tagsfound == 0) { /* found the 1st tag - initialize the return array */
 			array_init(return_value);
 		}
 
-		if (zend_hash_find(Z_ARRVAL_P(return_value), key, strlen(key) + 1, (void **) &element) == FAILURE) {
-			MAKE_STD_ZVAL(values);
-			array_init(values);
-			
-			zend_hash_update(Z_ARRVAL_P(return_value), key, strlen(key) + 1, (void *) &values, sizeof(zval*), (void **) &element);
-		} 
-			
-		add_next_index_stringl(*element, buffer+inx, len, 1);
+		if ((element = zend_hash_str_find(Z_ARRVAL_P(return_value), key, strlen(key))) == NULL) {
+			array_init(&values);
+
+			element = zend_hash_str_update(Z_ARRVAL_P(return_value), key, strlen(key), &values);
+		}
+
+		add_next_index_stringl(element, (char *) buffer+inx, len);
 		inx += len;
 		tagsfound++;
 	}
